@@ -1,17 +1,9 @@
 """ tests converted webapi """
 import os
-from datetime import datetime
 from unittest import TestCase, mock
-from requests import HTTPError
 
-import pytest
-from rich.console import Console
-
-import tidy3d as td
-from tidy3d.log import DataError
 import tidy3d.web as web
 from tidy3d.web.auth import get_credentials, encode_password
-from tidy3d.web.webapi import _query_or_create_folder
 from .utils import SIM_CONVERT as sim_original
 from .utils import clear_tmp
 
@@ -24,6 +16,9 @@ CALLBACK_URL = "https://callbackurl"
 
 task_id_global = []
 
+# from tidy3d.web.config import DEFAULT_CONFIG
+# DEFAULT_CONFIG.solver_version = "python_upgrade-0.0.0"
+
 
 def _get_gloabl_task_id():
     """returns the task id from the list"""
@@ -134,14 +129,6 @@ def test_query_folder_by_name():
     _ = web.webapi._query_or_create_folder("default")
 
 
-@clear_tmp
-def test_source_norm():
-    """test complete run"""
-    sim_data_raw = web.run(
-        simulation=sim_original, task_name="test_webapi", path=PATH_SIM_DATA, normalize_index=None
-    )
-
-
 """ Jobs """
 
 jobs_global = []
@@ -162,7 +149,6 @@ def test_job_0_run():
 def test_job_1_upload():
     """test that task uploads ok"""
     job = web.Job(simulation=sim_original, task_name="test_job")
-    job.upload()
     jobs_global.append(job)
 
 
@@ -208,14 +194,6 @@ def _test_job_7_delete():
     assert task_info.status in ("deleted", "deleting")
 
 
-def test_job_source_norm(caplog):
-    """test complete run"""
-    job = web.Job(simulation=sim_original, task_name="test_job", callback_url=CALLBACK_URL)
-    sim_data_norm = job.run(path=PATH_SIM_DATA, normalize_index=0)
-    with pytest.raises(DataError):
-        sim_data_norm = web.load(task_id=job.task_id, normalize_index=1)
-
-
 """ Batches """
 
 batches_global = []
@@ -240,7 +218,6 @@ def test_batch_1_upload():
     sims = 2 * [sim_original]
     simulations = {f"task_{i}": sims[i] for i in range(len(sims))}
     batch = web.Batch(simulations=simulations)
-    batch.upload()
     batches_global.append(batch)
 
 
@@ -269,7 +246,6 @@ def test_batch_5_download():
     batch.download(path_dir=PATH_DIR_SIM_DATA)
 
 
-@clear_tmp
 def test_batch_6_load():
     """load the results into sim_data"""
     batch = _get_gloabl_batch()
@@ -282,7 +258,9 @@ def test_batch_6_load():
 @clear_tmp
 def test_batchdata_7_load():
     """load the BatchData from file."""
-    sim_data_dict = BatchData.load(path_dir=PATH_DIR_SIM_DATA)
+    batch = _get_gloabl_batch()
+    batch.to_file(os.path.join(PATH_DIR_SIM_DATA, "batch.json"))
+    sim_data_dict = web.BatchData.load(path_dir=PATH_DIR_SIM_DATA)
 
 
 def _test_batch_8_delete():
