@@ -1,5 +1,6 @@
 # pylint: disable=too-many-lines, too-many-arguments
 """ Container holding all information about simulation and its components"""
+from __future__ import annotations
 from typing import Dict, Tuple, List, Set, Union
 from math import isclose
 
@@ -14,15 +15,15 @@ from .base import cached_property
 from .validators import assert_unique_names, assert_objects_in_sim_bounds
 from .validators import validate_mode_objects_symmetry
 from .geometry import Box
-from .types import Ax, Shapely, FreqBound, GridSize, Axis, annotate_type
+from .types import Ax, Shapely, FreqBound, Axis, annotate_type
 from .grid import Coords1D, Grid, Coords, GridSpec, UniformGrid
 from .medium import Medium, MediumType, AbstractMedium, PECMedium
-from .boundary import BoundarySpec, Symmetry, BlochBoundary, PECBoundary, PMCBoundary
+from .boundary import BoundarySpec, Symmetry, BlochBoundary, PECBoundary, PMCBoundary, Periodic
 from .boundary import PML, StablePML, Absorber
 from .structure import Structure
 from .source import SourceType, PlaneWave, GaussianBeam, AstigmaticGaussianBeam
 from .monitor import MonitorType, Monitor, FreqMonitor
-from .monitor import AbstractFieldMonitor
+from .monitor import AbstractFieldMonitor, DiffractionMonitor
 from .viz import add_ax_if_none, equal_aspect
 
 from .viz import MEDIUM_CMAP, PlotParams, plot_params_symmetry
@@ -30,7 +31,7 @@ from .viz import plot_params_structure, plot_params_pml, plot_params_override_st
 from .viz import plot_params_pec, plot_params_pmc, plot_params_bloch
 
 from ..version import __version__
-from ..constants import C_0, MICROMETER, SECOND, inf
+from ..constants import C_0, SECOND, inf
 from ..log import log, Tidy3dKeyError, SetupError, ValidationError
 from ..updater import Updater
 
@@ -107,13 +108,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         units=SECOND,
     )
 
-    grid_size: Union[GridSpec, Tuple[GridSize, GridSize, GridSize]] = pydantic.Field(
-        None,
-        title="Grid Size",
-        description="NOTE: 'grid_size' has been replaced by 'grid_spec'.",
-        units=MICROMETER,
-    )
-
     medium: MediumType = pydantic.Field(
         Medium(),
         title="Background Medium",
@@ -194,7 +188,8 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         title="Courant Factor",
         description="Courant stability factor, controls time step to spatial step ratio. "
         "Lower values lead to more stable simulations for dispersive materials, "
-        "but result in longer simulation times.",
+        "but result in longer simulation times. This factor is normalized to no larger than 1 "
+        "when CFL stability condition is met in 3D.",
         gt=0.0,
         le=1.0,
     )
@@ -207,18 +202,6 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
     """ Validating setup """
 
-    @pydantic.validator("grid_size", always=True)
-    def _error_use_grid_size(cls, val):
-        """If ``grid_size`` is provided, raise an error."""
-
-        if val is not None:
-            raise ValidationError(
-                "'grid_size' has been replaced by 'grid_spec'. See the "
-                "'GridSpec' documentation for more information."
-            )
-
-        return val
-
     @pydantic.validator("grid_spec", always=True)
     def _validate_auto_grid_wavelength(cls, val, values):
         """Check that wavelength can be defined if there is auto grid spec."""
@@ -437,6 +420,30 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
                 )
         return val
 
+    @pydantic.validator("monitors", always=True)
+    def diffraction_monitor_boundaries(cls, val, values):
+        """If any :class:`DiffractionMonitor` exists, ensure boundary conditions in the
+        transverse directions are periodic or Bloch."""
+        monitors = val
+        boundary_spec = values.get("boundary_spec")
+        for monitor in monitors:
+            if isinstance(monitor, DiffractionMonitor):
+                _, (n_x, n_y) = monitor.pop_axis(["x", "y", "z"], axis=monitor.normal_axis)
+                boundaries = [
+                    boundary_spec[n_x].plus,
+                    boundary_spec[n_x].minus,
+                    boundary_spec[n_y].plus,
+                    boundary_spec[n_y].minus,
+                ]
+                # make sure the transverse boundaries are either periodic or Bloch
+                for boundary in boundaries:
+                    if not isinstance(boundary, (Periodic, BlochBoundary)):
+                        raise SetupError(
+                            f"The 'DiffractionMonitor' {monitor.name} requires periodic "
+                            f"or Bloch boundaries along dimensions {n_x} and {n_y}."
+                        )
+        return val
+
     @pydantic.validator("grid_spec", always=True)
     def _warn_grid_size_too_small(cls, val, values):  # pylint:disable=too-many-locals
         """Warn user if any grid size is too large compared to minimum wavelength in material."""
@@ -1816,7 +1823,7 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
         return make_eps_data(coords)
 
     @classmethod
-    def from_file(cls, fname: str, **parse_kwargs):
+    def from_file(cls, fname: str, **parse_kwargs) -> Simulation:
         """Loads a :class:`Tidy3dBaseModel` from .yaml or .json file.
 
         Parameters
@@ -1838,4 +1845,4 @@ class Simulation(Box):  # pylint:disable=too-many-public-methods
 
         updater = Updater.from_file(fname)
         sim_dict = updater.update_to_current()
-        return Simulation.parse_obj(sim_dict, **parse_kwargs)
+        return cls.parse_obj(sim_dict, **parse_kwargs)
