@@ -1,12 +1,12 @@
 """Fit PoleResidue Dispersion models to optical NK data
 """
 
-from typing import Tuple, List, Optional
+from typing import Tuple, List, Optional, Union
 import csv
 import codecs
 import requests
 
-import nlopt
+import scipy.optimize as opt
 import numpy as np
 from rich.progress import Progress
 from pydantic import Field, validator
@@ -15,7 +15,7 @@ from ...components.base import Tidy3dBaseModel
 from ...components import PoleResidue, AbstractMedium
 from ...constants import C_0, HBAR, MICROMETER
 from ...components.viz import add_ax_if_none
-from ...components.types import Ax, Numpy, NumpyArray, ArrayLike
+from ...components.types import Ax, ArrayLike
 from ...log import log, ValidationError, WebError, SetupError
 
 
@@ -23,20 +23,20 @@ class DispersionFitter(Tidy3dBaseModel):
     """Tool for fitting refractive index data to get a
     dispersive medium described by :class:`.PoleResidue` model."""
 
-    wvl_um: ArrayLike = Field(
+    wvl_um: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
         ...,
         title="Wavelength data",
         description="Wavelength data in micrometers.",
         units=MICROMETER,
     )
 
-    n_data: ArrayLike = Field(
+    n_data: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
         ...,
         title="Index of refraction data",
         description="Real part of the complex index of refraction.",
     )
 
-    k_data: ArrayLike = Field(
+    k_data: Union[Tuple[float, ...], ArrayLike[float, 1]] = Field(
         None,
         title="Extinction coefficient data",
         description="Imaginary part of the complex index of refraction.",
@@ -79,7 +79,7 @@ class DispersionFitter(Tidy3dBaseModel):
 
     def _filter_wvl_range(
         self, wvl_min: float = None, wvl_max: float = None
-    ) -> Tuple[NumpyArray, NumpyArray, NumpyArray]:
+    ) -> Tuple[Tuple[float, ...], Tuple[float, ...], Tuple[float, ...]]:
         """
         Filter the wavelength-nk data to wavelength range [wvl_min,wvl_max]
         for fitting.
@@ -93,7 +93,7 @@ class DispersionFitter(Tidy3dBaseModel):
 
         Returns
         -------
-        Tuple[NumpyArray,NumpyArray,NumpyArray]
+        Tuple[Tuple[float, ...], Tuple[float, ...], Tuple[float, ...]]
             Filtered wvl_um, n_data, k_data
 
         """
@@ -142,12 +142,12 @@ class DispersionFitter(Tidy3dBaseModel):
         return AbstractMedium.nk_to_eps_complex(n=n_data, k=k_data)
 
     @property
-    def freqs(self) -> NumpyArray:
+    def freqs(self) -> Tuple[float, ...]:
         """Convert filtered input wavelength data to frequency.
 
         Returns
         -------
-        NumpyArray
+        Tuple[float, ...]
             Frequency array converted from filtered input wavelength data
         """
 
@@ -264,9 +264,8 @@ class DispersionFitter(Tidy3dBaseModel):
         """
         coeffs_scaled = coeffs / HBAR
         poles_a, poles_c = DispersionFitter._unpack_coeffs(coeffs_scaled)
-        poles = [(complex(a), complex(c)) for (a, c) in zip(poles_a, poles_c)]
         # poles = [((a.real, a.imag), (c.real, c.imag)) for (a, c) in zip(poles_a, poles_c)]
-        return poles
+        return [(complex(a), complex(c)) for (a, c) in zip(poles_a, poles_c)]
 
     @staticmethod
     def _poles_to_coeffs(poles):
@@ -402,7 +401,7 @@ class DispersionFitter(Tidy3dBaseModel):
             Results of single fit: (dispersive medium, RMS error).
         """
 
-        def constraint(coeffs, _grad):
+        def constraint(coeffs, _grad=None):
             """Evaluates the nonlinear stability criterion of
             Hongjin Choi, Jae-Woo Baek, and Kyung-Young Jung,
             "Comprehensive Study on Numerical Aspects of Modified
@@ -429,7 +428,7 @@ class DispersionFitter(Tidy3dBaseModel):
             res[res >= 0] = 0
             return np.sum(res)
 
-        def obj(coeffs, _grad):
+        def obj(coeffs, _grad=None):
             """objective function for fit
 
             Parameters
@@ -448,19 +447,13 @@ class DispersionFitter(Tidy3dBaseModel):
             medium = self._make_medium(coeffs)
             eps_model = medium.eps_model(self.freqs)
             residual = self.eps_data - eps_model
-            rms_error = np.sqrt(np.sum(np.square(np.abs(residual))) / len(self.eps_data))
             # cons = constraint(coeffs, _grad)
-            return rms_error
+            return np.sqrt(np.sum(np.square(np.abs(residual))) / len(self.eps_data))
 
         # set initial guess
         num_coeffs = num_poles * 4
         coeffs0 = 2 * (np.random.random(num_coeffs) - 0.5)
 
-        # set method and objective
-        method = nlopt.LN_NELDERMEAD
-        opt = nlopt.opt(method, num_coeffs)
-        opt.set_min_objective(obj)
-
         # set bounds
         bounds_upper = np.zeros(num_coeffs, dtype=float)
         bounds_lower = np.zeros(num_coeffs, dtype=float)
@@ -482,25 +475,26 @@ class DispersionFitter(Tidy3dBaseModel):
         bounds_lower[indices + 3] = -np.inf
         bounds_upper[indices + 3] = np.inf
 
-        opt.set_lower_bounds(bounds_lower.tolist())
-        opt.set_upper_bounds(bounds_upper.tolist())
+        bounds = list(zip(bounds_lower, bounds_upper))
 
-        # opt.add_inequality_constraint(constraint)
-        opt.set_xtol_rel(1e-5)
-        opt.set_ftol_rel(1e-5)
+        # TODO: set up constraint properly
+        scipy_constraint = opt.NonlinearConstraint(constraint, lb=0, ub=np.inf)
 
-        # run global optimization with opt as inner loop
-        optglob = nlopt.opt(nlopt.LN_AUGLAG, num_coeffs)
-        optglob.set_min_objective(obj)
-        optglob.set_lower_bounds(bounds_lower.tolist())
-        optglob.set_upper_bounds(bounds_upper.tolist())
-        optglob.add_inequality_constraint(constraint)
-        optglob.set_xtol_rel(1e-5)
-        optglob.set_maxeval(10000)
-        optglob.set_ftol_rel(1e-7)
-        optglob.set_local_optimizer(opt)
-        coeffs = optglob.optimize(coeffs0)
-        rms_error = optglob.last_optimum_value()
+        # TODO: set options properly
+        res = opt.minimize(
+            obj,
+            coeffs0,
+            args=(),
+            method="SLSQP",
+            bounds=bounds,
+            constraints=(scipy_constraint,),
+            tol=1e-7,
+            callback=None,
+            options=dict(maxiter=10000),
+        )
+
+        coeffs = res.x
+        rms_error = obj(coeffs)
 
         # set the latest fit
         medium = self._make_medium(coeffs)
@@ -510,7 +504,7 @@ class DispersionFitter(Tidy3dBaseModel):
     def plot(
         self,
         medium: PoleResidue = None,
-        wvl_um: Numpy = None,
+        wvl_um: Tuple[float, ...] = None,
         ax: Ax = None,
     ) -> Ax:
         """Make plot of model vs data, at a set of wavelengths (if supplied).
@@ -519,7 +513,7 @@ class DispersionFitter(Tidy3dBaseModel):
         ----------
         medium : :class:`.PoleResidue` = None
             medium containing model to plot against data
-        wvl_um : Numpy = None
+        wvl_um : Tuple[float, ...] = None
             Wavelengths to evaluate model at for plot in micrometers.
         ax : matplotlib.axes._subplots.Axes = None
             Axes to plot the data on, if None, a new one is created.
@@ -675,24 +669,21 @@ class DispersionFitter(Tidy3dBaseModel):
         for row in data_url[1:]:
             if has_k == 1:
                 k_lam.append([float(x) for x in row])
+            elif row[0] == "wl":
+                has_k += 1
             else:
-                if row[0] == "wl":
-                    has_k += 1
-                else:
-                    n_lam.append([float(x) for x in row])
+                n_lam.append([float(x) for x in row])
 
         n_lam = np.array(n_lam)
         k_lam = np.array(k_lam)
 
-        # for data containing k
         if has_k == 1:
-            # now let's make sure wvl_um in n_lam and k_lam match
-            if not np.allclose(n_lam[:, 0], k_lam[:, 0]):
-                raise ValidationError(
-                    "Invalid URL. Both n and k should be provided at each wavelength."
-                )
+            if np.allclose(n_lam[:, 0], k_lam[:, 0]):
+                return cls(wvl_um=n_lam[:, 0], n_data=n_lam[:, 1], k_data=k_lam[:, 1], **kwargs)
+            raise ValidationError(
+                "Invalid URL. Both n and k should be provided at each wavelength."
+            )
 
-            return cls(wvl_um=n_lam[:, 0], n_data=n_lam[:, 1], k_data=k_lam[:, 1], **kwargs)
         return cls(wvl_um=n_lam[:, 0], n_data=n_lam[:, 1], **kwargs)
 
     @classmethod
@@ -706,6 +697,27 @@ class DispersionFitter(Tidy3dBaseModel):
         **loadtxt_kwargs
             Kwargs passed to ``np.loadtxt``, such as ``skiprows``, ``delimiter``.
 
+        Hint
+        ----
+        The data file should be in this format (``delimiter`` and ``skiprows`` can be
+        customized in ``**loadtxt_kwargs``):
+
+        * For lossless media::
+
+            wl       n
+            [float] [float]
+            .       .
+            .       .
+            .       .
+
+        * For lossy media::
+
+            wl       n       k
+            [float] [float] [float]
+            .       .       .
+            .       .       .
+            .       .       .
+
         Returns
         -------
         :class:`DispersionFitter`
